<?php

namespace Drupal\kraken\Plugin\ImageAPIOptimizeProcessor;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Image\ImageFactory;
use Drupal\Core\Site\Settings;
use Drupal\imageapi_optimize\ConfigurableImageAPIOptimizeProcessorBase;
use Drupal\imageapi_optimize\ImageAPIOptimizeProcessorBase;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Optimize images using the Kraken.io webservice.
 *
 * @ImageAPIOptimizeProcessor(
 *   id = "kraken",
 *   label = @Translation("Kraken.io"),
 *   description = @Translation("Optimize images using the Kraken.io webservice.")
 * )
 */
class kraken extends ConfigurableImageAPIOptimizeProcessorBase {

  /**
   * The file system service.
   *
   * @var \Drupal\core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerInterface $logger, ImageFactory $image_factory, FileSystemInterface $file_system, ClientInterface $http_client) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger, $image_factory);

    $this->fileSystem = $file_system;
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('imageapi_optimize'),
      $container->get('image.factory'),
      $container->get('file_system'),
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'api_key' => NULL,
      'api_secret' => NULL,
      'lossy' => TRUE,
      'webp' => FALSE,
      'logging' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['api_key'] = array(
      '#title' => t('API Key'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $this->configuration['api_key'],
    );

    $form['api_secret'] = array(
      '#title' => t('API Secret'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $this->configuration['api_secret'],
    );

    $form['lossy'] = array(
      '#title' => t('Use lossy compression'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['lossy'],
    );

    $form['webp'] = array(
      '#title' => t('Convert to webp'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['webp'],
    );

    $form['logging'] = array(
      '#title' => $this->t('Enable Success Logging'),
      '#type' => 'checkbox',
      '#description' => $this->t('If checked, every single optimize success will be logged in the watchdog.'),
      '#default_value' => $this->configuration['logging'],
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['api_key'] = $form_state->getValue('api_key');
    $this->configuration['api_secret'] = $form_state->getValue('api_secret');
    $this->configuration['lossy'] = $form_state->getValue('lossy');
    $this->configuration['webp'] = $form_state->getValue('webp');
    $this->configuration['logging'] = $form_state->getValue('logging');
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $description = '';

    if (!class_exists('\Kraken')) {
      $description .= $this->t('<strong>Could not locate Kraken PHP library.</strong>');
    }
    else {
      if ($this->configuration['lossy']) {
        $description .= $this->t('Using lossy compression.');
      }
      else {
        $description .= $this->t('Using lossless compression.');
      }
      if ($this->configuration['webp']) {
        $description .= $this->t(' Convert to webp.');
      }
      if ($this->configuration['logging']) {
        $description .= $this->t(' Success Logging enabled.');
      }
      else {
        $description .= $this->t(' Success logging disabled.');
      }
    }

    $summary = array(
      '#markup' => $description,
    );
    $summary += parent::getSummary();

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function applyToImage($image_uri) {

    if (class_exists('\Kraken')) {
      if (!empty($this->configuration['api_key']) && !empty($this->configuration['api_secret'])) {
        $proxy_params = [];
        $proxy_settings = Settings::get('http_client_config', []);
        if (!empty($proxy_settings['proxy']['https'])) {
          $proxy_params['proxy'] = $proxy_settings['proxy']['https'];
        }
        // @todo: Make this timeout configurable.
        $timeout = 5;
        $kraken = new \Kraken($this->configuration['api_key'], $this->configuration['api_secret'], $timeout, $proxy_params);
        $params = array(
          'file' => $this->fileSystem->realpath($image_uri),
          'wait' => TRUE,
          'lossy' => (bool) $this->configuration['lossy'],
          'webp' => (bool) $this->configuration['webp'],
        );

        // Send the request to Kraken.
        $data = $kraken->upload($params);

        if (!empty($data['success']) && !empty($data['kraked_url'])) {
          try {
            $krakedFile = $this->httpClient->get($data['kraked_url']);
            if ($krakedFile->getStatusCode() == 200) {
              $this->fileSystem->saveData($krakedFile->getBody(), $image_uri, FileSystemInterface::EXISTS_REPLACE);
              if ($this->configuration['logging']) {
                $this->logger->info('@file_name was successfully processed by Kraken.io.
        Original size: @original_size; Kraked size: @kraked_size; Total saved:
        @saved_bytes. All figures in bytes', [
                    '@file_name' => $image_uri,
                    '@original_size' => $data['original_size'],
                    '@kraked_size' => $data['kraked_size'],
                    '@saved_bytes' => $data['saved_bytes'],
                  ]
                );
              }
              return TRUE;
            }
          } catch (RequestException $e) {
            $this->logger->error('Failed to download optimized image using Kraken.io due to "%error".', array('%error' => $e->getMessage()));
          }
        }
        else {
          $this->logger->error('Kraken.io could not optimize the uploaded image.');
        }
      }
      else {
        $this->logger->error('Kraken API key or secret not set.');
      }
    }
    else {
      $this->logger->error('Could not locate Kraken PHP library.');
    }
    return FALSE;
  }
}
